# Program keyer1.py (load as main.py for auto-start)
# Designed to work on the PMK board by Dave Benson K1SWL and George Heron N2APB
# This is a basic, simple keyer with no extras like memory or commands
# It's simplicity should help with both understanding how it operates, and for making changes.
# Released to the Public Domain - 31 August 2023 Steve Beckman N3SB

from machine import Pin, PWM, ADC
from time import sleep, sleep_ms

# Keyer Mode - Iambic A or B
Keyer_Mode = "B"

# LED Pin Defines
D2 = 9
D3 = 11
D4 = 13
D5 = 15
D6 = 7  # Record LED

# Output Pin Definitions
Keyer = 18

SpeedPin = 26

SideTonePin = 20

LED_D2 = Pin(D2, Pin.OUT)
LED_D3 = Pin(D3, Pin.OUT)
LED_D4 = Pin(D4, Pin.OUT)
LED_D5 = Pin(D5, Pin.OUT)
LED_D6 = Pin(D6, Pin.OUT)
Keyer_Out = Pin(Keyer, Pin.OUT)

SideTone = PWM(Pin(SideTonePin))
SideTone.freq(600)

SpeedPot = ADC(SpeedPin)

# Keyer Input Pin Definitions
Dit_Pin = 17
Dah_Pin = 16

# Button Input Pin Definitions
Record_Button = 6
Message1_Button = 8
Message2_Button = 10
RPT_Button = 12
DECR_Button = 14
Jumper = 5

# Turn all the LEDs to off
LED_D2.off()
LED_D3.off()
LED_D4.off()
LED_D5.off()
LED_D6.off()

# Turn the Keyer Output off
Keyer_Out.off()

# Configure key input and button input lines
Dit_Input = Pin(Dit_Pin, Pin.IN, Pin.PULL_UP)
Dah_Input = Pin(Dah_Pin, Pin.IN, Pin.PULL_UP)
Record_Button_Input = Pin(Record_Button, Pin.IN, Pin.PULL_UP)
Message1_Button_Input = Pin(Message1_Button, Pin.IN, Pin.PULL_UP)
Message2_Button_Input = Pin(Message2_Button, Pin.IN, Pin.PULL_UP)
RPT_Button_Input = Pin(RPT_Button, Pin.IN, Pin.PULL_UP)
DECR_Button_Input = Pin(DECR_Button, Pin.IN, Pin.PULL_UP)
Jumper_Input = Pin(Jumper, Pin.IN, Pin.PULL_UP)

# Initialize sidetone to off
SideTone.duty_u16(0)

# Initialize Interrupt Flag Variables
Dit_Int_Flag = 0
Dah_Int_Flag = 0
Record_Int_Flag = 0
Message1_Int_Flag = 0
Message2_Int_Flag = 0    
RPT_Int_Flag = 0    
DECR_Int_Flag = 0    

# Interrupt Callback Functions
def dit_callback(pin):
    global Dit_Int_Flag
    Dit_Int_Flag = 1

def dah_callback(pin):
    global Dah_Int_Flag
    Dah_Int_Flag = 1

def Record_Button_callback(pin):
    global Record_Int_Flag
    print("Record")
    Record_Int_Flag = 1

def Message1_Button_callback(pin):
    global Message1_Int_Flag
    print("Message 1")
    Message1_Int_Flag = 1

def Message2_Button_callback(pin):
    global Message2_Int_Flag
    print("Message 2")
    Message2_Int_Flag = 1

def RPT_Button_callback(pin):
    global RPT_Int_Flag
    print("RPT")
    RPT_Int_Flag = 1

def DECR_Button_callback(pin):
    global DECR_Int_Flag
    print("DECR")
    DECR_Int_Flag = 1

# Other Functions
def Send_Dit(delay):
    Keyer_Out.on()
    SideTone.duty_u16(32767)
    sleep(delay)
    Keyer_Out.off()
    SideTone.duty_u16(0)
    sleep(delay)    

def Send_Dah(delay):
    Keyer_Out.on()
    SideTone.duty_u16(32767)
    sleep(delay * 3)
    Keyer_Out.off()
    SideTone.duty_u16(0)
    sleep(delay)
    
def Get_Speed():
    # Read the CW Speed pot, calculate the element delay
    Speed = int((SpeedPot.read_u16() - 208)*35/65135)+5
    CW_Delay = 1.2 / Speed
    return CW_Delay

def Poll_Dit(): # Reads the status of the dit paddle input
    if Dit_Input() == 0:
        Dit_Pressed = 1
    else:
        Dit_Pressed = 0
    return Dit_Pressed

def Poll_Dah(): # Reads the status of the dah paddle input
    if Dah_Input() == 0:
        Dah_Pressed = 1
    else:
        Dah_Pressed = 0
    return Dah_Pressed

# Create the interrupts
Dit_Input.irq(trigger = Pin.IRQ_FALLING, handler = dit_callback)
Dah_Input.irq(trigger = Pin.IRQ_FALLING, handler = dah_callback)
Record_Button_Input.irq(trigger = Pin.IRQ_FALLING, handler = Record_Button_callback)
Message1_Button_Input.irq(trigger = Pin.IRQ_FALLING, handler = Message1_Button_callback)
Message2_Button_Input.irq(trigger = Pin.IRQ_FALLING, handler = Message2_Button_callback)
RPT_Button_Input.irq(trigger = Pin.IRQ_FALLING, handler = RPT_Button_callback)
DECR_Button_Input.irq(trigger = Pin.IRQ_FALLING, handler = DECR_Button_callback)

Sent_Dit = 0
Sent_Dah = 0

while True:
    CW_Delay = Get_Speed()
    Dit = Poll_Dit()
    Dah = Poll_Dah()
    
    while Dit == 1 or Dah == 1 or Dit_Int_Flag == 1 or Dah_Int_Flag == 1:
        if (Dit == 1 and Sent_Dit == 0) or Dit_Int_Flag == 1:
            Send_Dit(CW_Delay)
            Dit_Int_Flag = 0
            Dit = 0
            if Keyer_Mode == "A":
                Dah = 0
            Sent_Dit = 1

        else:
            Sent_Dit = 0
        
        if (Dah == 1 and Sent_Dah == 0) or Dah_Int_Flag == 1:
            Send_Dah(CW_Delay)
            Dah_Int_Flag = 0
            Dah = 0
            if Keyer_Mode == "A":
                Dit = 0
            Sent_Dah = 1

        else:
            Sent_Dah = 0
            

            


